require 'yaml'

class Tikici
  def initialize(config_file = 'tiki.yml')
    @config_file = config_file
  end

	def symbolize(obj)
		if obj.is_a? Hash
			return obj.inject({}) do |memo, (k, v)|
				memo.tap { |m| m[k.to_sym] = symbolize(v) }
			end
		elsif obj.is_a? Array
			return obj.map { |memo| symbolize(memo) }
		end
		obj
	end

  def self.start
    Tikici.new.exec
  end

  def config
    @config ||= symbolize(YAML.load_file(@config_file))
  rescue
    {}
  end

  def dependencies
    @dependencies ||= Dependencies.new(config.fetch(:dependencies, nil))
  end

  def test
    @test ||= Test.new(config.fetch(:test, nil))
  end

  def deployment
    @deployment ||= Deployment.new(config.fetch(:deployment, nil))
  end
    
  def exec
    dependencies.exec
    test.exec
    deployment.exec
  end
end

class Deployment
  def initialize(config)
    @config = config
  end
  
  def environments
    {}.tap do |result|
      @config.each do |key, val| 
        result[key.to_sym] = DeploymentEnv.new(val)
      end unless @config.nil?
    end
  end

  def environment_values
    environments.values
  end

  def exec
    environment_values.map(&:exec)
  end
end

class DeploymentEnv
  attr_accessor :branch, :commands

  def initialize(config)
    @config = config
  end

  def exec
    commands.map(&:exec) if current_branch =~ Regexp.new(branch) unless branch.nil?
  end

  def current_branch
    `echo ${GIT_BRANCH#*/}`
  end
  
  def branch
    @branch ||= @config.fetch(:branch, nil) unless @config.nil?
  end

  def commands
    @config.fetch(:commands, []).map {|cmd| Command.new(cmd)} 
  rescue
    []
  end
end

class Test
  def initialize(config)
    @config = config
  end

  def commands
    @config.fetch(:commands, []).map{|cmd| Command.new(cmd)} 
  rescue
    []
  end

  def post_commands
    @config.fetch(:post, []).map{|cmd| Command.new(cmd)} 
  rescue
    []
  end

  def exec
    commands.map(&:exec)
    post_commands.map(&:exec)
  end
end

class Dependencies
  def initialize(config)
    @config = config
  end

  def commands
    @config.fetch(:commands, []).map{|cmd| Command.new(cmd)} 
  rescue
    []
  end

  def exec
    commands.map(&:exec)
  end
end

class Command   
  def initialize(cmd)
    @cmd = cmd
  end

  def exec
    Kernel.system(@cmd) or abort
  end
end
